#THIS IS timediffz 1.0.0.2 (fractionals mode) - made by wedu_official
import sys
# noinspection PyUnresolvedReferences
from fractionals import Fraction #cython
import requests, bisect
sys.set_int_max_str_digits(0)
TT_TAI_OFFSET_SECONDS = Fraction(32184, 1000)
_LEAP_SECOND_DAYNUMS = [Fraction(4882999, 2), Fraction(4883367, 2), Fraction(4884097, 2), Fraction(4884827, 2), Fraction(4885557, 2), Fraction(4886289, 2), Fraction(4887019, 2), Fraction(4887749, 2), Fraction(4888479, 2), Fraction(4889573, 2), Fraction(4890303, 2), Fraction(4891033, 2), Fraction(4892495, 2), Fraction(4894323, 2), Fraction(4895785, 2), Fraction(4896515, 2), Fraction(4897609, 2), Fraction(4898339, 2), Fraction(4899069, 2), Fraction(4900167, 2), Fraction(4901261, 2), Fraction(4902359, 2), Fraction(4907473, 2), Fraction(4909665, 2), Fraction(4912219, 2), Fraction(4914409, 2), Fraction(4915509, 2)]
_MONTH_MAP = {'Jan':1,'Feb':2,'Mar':3,'Apr':4,'May':5,'Jun':6,'Jul':7,'Aug':8,'Sep':9,'Oct':10,'Nov':11,'Dec':12}
def _day_number(y, m, d, h=0, mi=0, s=0, sub_seconds=0, sub_unit_depth=10**6):
    # Compute Julian day number for date
    a = (14 - m) // 12
    y_adj = y + 4800 - a
    m_adj = m + 12*a - 3
    jd_integer = d + ((153 * m_adj + 2) // 5) + 365 * y_adj + y_adj // 4 - y_adj // 100 + y_adj // 400
    jd = Fraction(jd_integer) - Fraction(64091, 2)  # Equivalent to 32045.5
    # Add fractional day
    frac_day = Fraction(h*3600 + mi*60 + s, 86400) + Fraction(sub_seconds, sub_unit_depth * 86400) - Fraction(1,2)
    return Fraction(jd) + frac_day
def _get_online_daynums(dnr=False):
    try:
        r = requests.get("https://data.iana.org/time-zones/tzdb/leap-seconds.list", timeout=10)
        r.raise_for_status()
    except requests.RequestException: return None
    daynums = []
    for line in r.text.splitlines():
        if line and line[0].isdigit():
            parts = line.split()
            if len(parts) >= 6:
                day = int(parts[3])
                month_str = parts[4]
                year = int(parts[5])
                if month_str in _MONTH_MAP:
                    month = _MONTH_MAP[month_str]
                    dn = _day_number(year, month, day)
                    if dnr: return dn
                    bisect.insort_right(daynums, dn)
    return daynums[1:]
def _count_ls(y, m, d, UTDLS=False):
    dn = _day_number(y, m, d)
    if UTDLS:
        daynums = _get_online_daynums()
        # Fallback to hardcoded list if online fetch fails
        if daynums is None or not daynums:   # Check for None or an empty list
            daynums = _LEAP_SECOND_DAYNUMS
    else:
        daynums = _LEAP_SECOND_DAYNUMS
    return bisect.bisect_right(daynums, dn)
def _utc_to_tai_offset_seconds(y, m, d, UTDLS=False): return int(_count_ls(y, m, d, UTDLS=UTDLS) + 10)
def _tt_minus_tai_seconds(): return TT_TAI_OFFSET_SECONDS
def _tai_to_otherformats(count_ls_param=False, output_tt=False, count_ls_param_pra=None):
    if count_ls_param_pra is None: count_ls_param_pra = [0, 0, 0, True]
    if count_ls_param: return - (_count_ls(count_ls_param_pra[0], count_ls_param_pra[1], count_ls_param_pra[2], UTDLS=count_ls_param_pra[3]) + 10)  # add this to TAI to get UTC (integer)
    elif output_tt:
        return TT_TAI_OFFSET_SECONDS  # Fraction to add to TAI to get TT
    else: return 0
def _timestampinner(y, m, d, h=0, mi=0, s=0, sub_seconds=0, sub_unit_depth=10**6,
                    count_ls_param=True, UTDLS=False,
                    input_utc=True, input_tt=False, output_tt=False,
                    unix_epoch=False, custom_epoch_as_utc=True,custom_epoch_as_tt=False, custom_epoch=None):
    jd_input = Fraction(_day_number(y, m, d, h, mi, s, sub_seconds, sub_unit_depth))
    if unix_epoch: jd_epoch = Fraction(_day_number(1970, 1, 1))
    elif custom_epoch is not None: jd_epoch = Fraction(_day_number(*custom_epoch))
    else: jd_epoch = Fraction(0)
    base_seconds = (jd_input - jd_epoch) * 86400  # Fraction
    if input_utc and input_tt:
        raise ValueError("invaild input, it's impossible to decide if the input is utc or tt")
    if custom_epoch_as_utc and custom_epoch_as_tt:
        raise ValueError("invaild input, it's impossible to decide if the custom epoch is utc or tt")
    if count_ls_param and output_tt:
        raise ValueError("invaild input, it's impossible to decide if the output must be utc or tt")
    if input_utc: input_offset = Fraction(_utc_to_tai_offset_seconds(y, m, d, UTDLS=UTDLS))
    elif input_tt:input_offset = _tt_minus_tai_seconds()*-1
    else: input_offset = Fraction(0)
    epoch_offset = Fraction(0)
    if custom_epoch is not None:
        if custom_epoch_as_utc: epoch_offset = Fraction(_utc_to_tai_offset_seconds(custom_epoch[0], custom_epoch[1], custom_epoch[2], UTDLS=UTDLS))
        elif custom_epoch_as_tt: epoch_offset = Fraction(_utc_to_tai_offset_seconds(custom_epoch[0], custom_epoch[1], custom_epoch[2], UTDLS=UTDLS))
    tai_seconds = base_seconds + input_offset - epoch_offset
    out_offset = Fraction(_tai_to_otherformats(count_ls_param=count_ls_param, output_tt=output_tt, count_ls_param_pra=[y, m, d, UTDLS]))
    result_seconds = tai_seconds + out_offset
    return result_seconds
def _days_in_month(y, m):
    if m == 2:
        is_leap = (y % 4 == 0 and y % 100 != 0) or (y % 400 == 0)
        return 29 if is_leap else 28
    elif m in (4, 6, 9, 11):
        return 30
    else:
        return 31
def _monthscalner(s, e):
    is_negative = False
    if (s[0], s[1], s[2]) > (e[0], e[1], e[2]):
        is_negative = True
        s, e = e, s
    total_months = (e[0] - s[0]) * 12 + (e[1] - s[1])
    # Adjust if the end day is before the start day
    if e[2] < s[2]:
        total_months -= 1
        days_in_end_month = _days_in_month(e[0], e[1])
        day_fraction = Fraction(e[2] + (days_in_end_month - s[2]), days_in_end_month)
    else:
        # The fraction is simply the difference in days over the total days in the start month
        days_in_start_month = _days_in_month(s[0], s[1])
        day_fraction = Fraction(e[2] - s[2], days_in_start_month)
    result = Fraction(total_months) + day_fraction
    if is_negative:
        result *= -1
    return result
def _yearscalner(s, e):
    is_negative = False
    if (s[0], s[1], s[2]) > (e[0], e[1], e[2]):
        is_negative = True
        s, e = e, s
    total_years = e[0] - s[0]
    if (e[1], e[2]) < (s[1], s[2]): total_years -= 1
    total_months = (e[0] - s[0]) * 12 + (e[1] - s[1])
    remaining_months = total_months % 12
    if e[2] < s[2]: remaining_months -= 1
    days_in_start_month = _days_in_month(s[0], s[1])
    day_fraction_of_month = Fraction(e[2] - s[2], days_in_start_month)
    fractional_months = remaining_months + day_fraction_of_month
    result = Fraction(total_years) + Fraction(fractional_months, 12)
    if is_negative:
        result *= -1
    return result
def _time_difference(start, end, unit_factor, tz_s=0, tz_e=0, input_utc=True,output_tt=False,count_ls_param=True, UTDLS=False):
    start_tai = _timestampinner(*start, count_ls_param=count_ls_param, input_utc=input_utc, input_tt=False,output_tt=False, UTDLS=UTDLS)
    end_tai = _timestampinner(*end, count_ls_param=count_ls_param, input_utc=input_utc, input_tt=False, output_tt=False,UTDLS=UTDLS)
    delta = end_tai - start_tai - (tz_e - tz_s) * 3600
    if output_tt: delta = delta + TT_TAI_OFFSET_SECONDS
    return delta / Fraction(1,unit_factor)
def _seconds_in_year(y=None, custom_year=None, leap_year=None,uniform_year=False,tropical_year=False):
    if tropical_year: return Fraction(Fraction(3652421896698,10000000000) * 86400)
    if custom_year is not None: return custom_year
    if uniform_year: return Fraction(Fraction(3652425,10000) * 86400)
    if leap_year is None:
        if y is None: raise ValueError("Need year y or leap_year/custom_year/tropical_year/preleptic to compute seconds")
        leap_year = (y % 4 == 0 and (y % 100 != 0 or y % 400 == 0))
    return (366 if leap_year else 365) * 86400
def _get_year_seconds(y, leap_year=None, uniform_year=False, tropical_year=False, custom_year=None):
    return _seconds_in_year(y=y, leap_year=leap_year, uniform_year=uniform_year, tropical_year=tropical_year, custom_year=custom_year)
def _seconds_in_month(y, m):
    start = _timestampinner(y, m, 1, 0, 0, 0) # Assuming 0,0,0 as standard start time
    end = _timestampinner(y, m, _days_in_month(y, m), 23, 59, 59) # End of last second of the month,
    return end - start
def _get_month_seconds(y, m, month_unit=0, leap_year=False):
    if month_unit == 0:
        return _seconds_in_month(y, m)
    else:
        ref_month = month_unit
        ref_year = 3  # arbitrary reference year
        if leap_year:
            ref_year = 4
        return _seconds_in_month(ref_year, ref_month)
def is_leap(year, leap_year=False):
    if year is not None:
        return ((year % 4 == 0 and year % 100 != 0) or (year % 400 == 0))
    else:
        return leap_year
def _months_piecewise(s, e, tz_s=0, tz_e=0,
                      month_unit=0, leap_year=False,
                      input_utc=True, output_tt=False,
                      UTDLS=False, count_ls_param=True,
                      use_date_in_leapyears_checking=False,
                      year_for_leapyears_checking=None,
                      reference_year_for_scaling=None):
    """Compute difference between two timestamps in months (piecewise), fully continuous (first, mid, last)."""

    neg = False
    if s > e:
        s, e = e, s
        tz_s, tz_e = tz_e, tz_s
        neg = True

    # Same month → direct fraction
    if (s[0], s[1]) == (e[0], e[1]):
        denom = _get_month_seconds(
            s[0], s[1], month_unit,
            is_leap(
                year=(s[0] if use_date_in_leapyears_checking else year_for_leapyears_checking),
                leap_year=leap_year
            )
        )
        num = _time_difference(
            s, e, unit_factor=1,
            tz_s=tz_s, tz_e=tz_e,
            input_utc=input_utc, output_tt=output_tt,
            count_ls_param=count_ls_param, UTDLS=UTDLS
        )
        return -num / denom if neg else num / denom

    # === First segment (from s → end of its month) ===
    end_s = (s[0], s[1], _days_in_month(s[0], s[1]), 23, 59, 59)
    denom_first = _get_month_seconds(
        s[0], s[1], month_unit,
        is_leap(
            year=(s[0] if use_date_in_leapyears_checking else year_for_leapyears_checking),
            leap_year=leap_year
        )
    )
    first_part = _time_difference(
        s, end_s, unit_factor=1,
        tz_s=tz_s, tz_e=tz_s,
        input_utc=input_utc, output_tt=output_tt,
        count_ls_param=count_ls_param, UTDLS=UTDLS
    ) / denom_first

    # === Last segment (from start of its month → e) ===
    start_e = (e[0], e[1], 1, 0, 0, 0)
    denom_last = _get_month_seconds(
        e[0], e[1], month_unit,
        is_leap(
            year=(e[0] if use_date_in_leapyears_checking else year_for_leapyears_checking),
            leap_year=leap_year
        )
    )
    last_part = _time_difference(
        start_e, e, unit_factor=1,
        tz_s=tz_e, tz_e=tz_e,
        input_utc=input_utc, output_tt=output_tt,
        count_ls_param=count_ls_param, UTDLS=UTDLS
    ) / denom_last
    if s[1] == 12:
        mid_start = (s[0] + 1, 1, 1, 0, 0, 0)
    else:
        mid_start = (s[0], s[1] + 1, 1, 0, 0, 0)
    mid_end = (e[0], e[1], 1, 0, 0, 0)
    mid_part = Fraction(0)
    leap_correction = Fraction(0)
    if mid_start < mid_end:
        full_months = (mid_end[0] - mid_start[0]) * 12 + (mid_end[1] - mid_start[1])
        mid_part = Fraction(full_months)
        feb29_count = 0
        if full_months > 0 and not leap_year:  # Only count if using actual leap years
            def count_leap_years(start, end):
                """Count leap years from start to end inclusive."""
                def leaps_before(year):
                    return year // 4 - year // 100 + year // 400
                return leaps_before(end) - leaps_before(start - 1)
            start_year, start_month = mid_start[0], mid_start[1]
            end_year, end_month = mid_end[0], mid_end[1]
            # Adjust range based on month boundaries
            effective_start = start_year if start_month <= 2 else start_year + 1
            effective_end = end_year if end_month >= 2 else end_year - 1
            if effective_start <= effective_end:
                feb29_count = count_leap_years(effective_start, effective_end)
        if feb29_count:
            # reference month to convert days -> months (same logic you used previously)
            ref_y = reference_year_for_scaling if reference_year_for_scaling is not None else s[0]
            ref_m = month_unit if month_unit != 0 else s[1]
            # compute correct leap/ reference-year logic as you intend:
            year_to_check_for_ref = ref_y if not use_date_in_leapyears_checking else (
                        year_for_leapyears_checking or ref_y)
            denom_ref = _get_month_seconds(
                ref_y, ref_m, month_unit,
                is_leap(year=year_to_check_for_ref, leap_year=leap_year))
            # each Feb-29 is 86400 seconds; convert to months using denom_ref
            leap_correction = Fraction((feb29_count * 86400) / Fraction(denom_ref))
    # finally: mid_part + leap_correction
    mid_part = Fraction(mid_part) + Fraction(leap_correction)
    result = Fraction(first_part) + Fraction(mid_part) + Fraction(last_part)
    return result*-1 if neg else result


def _years_piecewise(s, e, tz_s=0, tz_e=0,
                     leap_year=None, uniform_year=False, tropical_year=False, custom_year=None,
                     input_utc=True, output_tt=False,
                     UTDLS=False, count_ls_param=True,year_for_leapyear_checking=None,lock_range_to_reference_year=False):
    neg = False
    if s > e:
        s, e = e, s
        tz_s, tz_e = tz_e, tz_s
        neg = True
    # Same year
    if s[0] == e[0]:
        denom = _get_year_seconds(s[0] if year_for_leapyear_checking is None else year_for_leapyear_checking, leap_year=leap_year, uniform_year=uniform_year,
                                  tropical_year=tropical_year, custom_year=custom_year)
        num = _time_difference(s, e, unit_factor=1, tz_s=tz_s, tz_e=tz_e,
                               input_utc=input_utc, output_tt=output_tt,
                               count_ls_param=count_ls_param, UTDLS=UTDLS)
        return -num / denom if neg else num / denom
    # Fraction of first year
    end_s = (s[0], 12, 31, 23, 59, 59)
    denom_first = _get_year_seconds(s[0] if year_for_leapyear_checking is None else year_for_leapyear_checking, leap_year=leap_year, uniform_year=uniform_year,
                                    tropical_year=tropical_year, custom_year=custom_year)
    first_part = _time_difference(s, end_s, unit_factor=1, tz_s=tz_s, tz_e=tz_s,
                                  input_utc=input_utc, output_tt=output_tt,
                                  count_ls_param=count_ls_param, UTDLS=UTDLS) / denom_first
    # Fraction of last year
    start_e = (e[0], 1, 1, 0, 0, 0)
    denom_last = _get_year_seconds(e[0] if year_for_leapyear_checking is None else year_for_leapyear_checking, leap_year=leap_year, uniform_year=uniform_year,
                                   tropical_year=tropical_year, custom_year=custom_year)
    last_part = _time_difference(start_e, e, unit_factor=1, tz_s=tz_e, tz_e=tz_e,
                                 input_utc=input_utc, output_tt=output_tt,
                                 count_ls_param=count_ls_param, UTDLS=UTDLS) / denom_last
    if lock_range_to_reference_year:
        full = 0  # When locked to reference year, no full years in between
    else:
        full = max(0, e[0] - s[0] - 1)  # Full years between (exclusive)
    result = first_part + full + last_part
    return result*-1 if neg else result


from setuptools import setup, Extension
from Cython.Build import cythonize

extensions = [
    Extension(
        name="fractionals",
        sources=["fractionals.pyx"],
        language="c++",
        libraries=["gmp"],  # link against GMP
        extra_compile_args=["-std=c++17"],  # optional, C++ standard
    )
]

setup(
    name="Fraction",
    ext_modules=cythonize(extensions, compiler_directives={"language_level": "3"}),
)



import warnings
try: import td1
except Exception: warnings.warn(f"timediffz 1.0.0.2 (fractions mode) failed to load correctly :(, using any fractions mode related function WOULD give an error and fall because of this, the error")
try: import td2
except Exception: warnings.warn(f"timediffz 1.0.0.2 (fractionals mode) failed to load correctly :(, using any fractionals mode related function WOULD give an error and fall because of this, the error")
def pdiff (s,e,tz_s=0,tz_e=0,unit_factor=1,count_ls_param=True,UTDLS=False,input_utc=True,output_tt=False):
    """this function give the difference between the two dates(s and e) in unit_factor, fractions version
    S: is the first date(starting date) and it's a list made off: [Y(year),M(month),D(day),H(hour),MI(minute),S(second),SS(sub-seconds)] and all of these items integers
    E: is the second date(ending date) and it has same format and data type as S
    tz_s: is the timezone where the S date was measured in by default library is set to use greenwich(england) time, and it's an integer value
    tz_e: is the same as tz_s in everything, but it's for E date
    unit_factor: this is the unit to be used, basically the difference between dates would be done in seconds , so you can if you want to get result in minutes for example
    to do that , make it into 60 and the difference would be divided by 60 leading to result being in the unit of minutes, and it's an integer
    count_ls_param: enabling this would count leap-seconds, boolean
    UTDLS: enabling this is useless in all cases except when count_ls_param is True, and it would fetch a list of updated leap seconds from the internet, boolean
    input_utc: this one is by default true, which is when your input is in the UTC system, if it's not make this parameter false
    and your dates would be considered as TAI input, boolean
    output_tt: this one is by default false, which is when you wanna your output to be in TT, boolean
    """
    return td1._time_difference(s, e, tz_s=tz_s, tz_e=tz_e, unit_factor=unit_factor, input_utc=input_utc, count_ls_param=count_ls_param, UTDLS=UTDLS, output_tt=output_tt)
def cdiff (s,e,tz_s=0,tz_e=0,unit_factor=1,count_ls_param=True,UTDLS=False,input_utc=True,output_tt=False):
    """this function give the difference between the two dates(s and e) in unit_factor, fractionals version
    S: is the first date(starting date) and it's a list made off: [Y(year),M(month),D(day),H(hour),MI(minute),S(second),SS(sub-seconds)] and all of these items integers
    E: is the second date(ending date) and it has same format and data type as S
    tz_s: is the timezone where the S date was measured in by default library is set to use greenwich(england) time, and it's an integer value
    tz_e: is the same as tz_s in everything, but it's for E date
    unit_factor: this is the unit to be used, basically the difference between dates would be done in seconds , so you can if you want to get result in minutes for example
    to do that , make it into 60 and the difference would be divided by 60 leading to result being in the unit of minutes, and it's an integer
    count_ls_param: enabling this would count leap-seconds, boolean
    UTDLS: enabling this is useless in all cases except when count_ls_param is True, and it would fetch a list of updated leap seconds from the internet, boolean
    input_utc: this one is by default true, which is when your input is in the UTC system, if it's not make this parameter false
    and your dates would be considered as TAI input, boolean
    output_tt: this one is by default false, which is when you wanna your output to be in TT, boolean
    """
    return td2._time_difference(s, e, tz_s=tz_s, tz_e=tz_e, unit_factor=unit_factor, input_utc=input_utc, count_ls_param=count_ls_param, UTDLS=UTDLS, output_tt=output_tt)
def pdseconds (s,e,tz_s=0,tz_e=0,count_ls_param=True,UTDLS=False,input_utc=True,output_tt=False):
    """this function give the difference between the two dates(s and e) in seconds, fractions version
    S: is the first date(starting date) and it's a list made off: [Y(year),M(month),D(day),H(hour),MI(minute),S(second),SS(sub-seconds)] and all of these items integers
    E: is the second date(ending date) and it has same format and data type as S
    tz_s: is the timezone where the S date was measured in by default library is set to use greenwich(england) time, and it's an integer value
    tz_e: is the same as tz_s in everything, but it's for E date
    count_ls_param: enabling this would count leap-seconds, boolean
    UTDLS: enabling this is useless in all cases except when count_ls_param is True, and it would fetch a list of updated leap seconds from the internet, boolean
    input_utc: this one is by default true, which is when your input is in the UTC system, if it's not make this parameter false
    and your dates would be considered as TAI input, boolean
    output_tt: this one is by default false, which is when you wanna your output to be in TT, boolean
    """
    return td1._time_difference(s, e, tz_s=tz_s, tz_e=tz_e, unit_factor=1, input_utc=input_utc, count_ls_param=count_ls_param, UTDLS=UTDLS, output_tt=output_tt)
def cdseconds(s, e, tz_s=0, tz_e=0, count_ls_param=True, UTDLS=False, input_utc=True, output_tt=False):
    """this function give the difference between the two dates(s and e) in seconds, fractionals version
        S: is the first date(starting date) and it's a list made off: [Y(year),M(month),D(day),H(hour),MI(minute),S(second),SS(sub-seconds)] and all of these items integers
        E: is the second date(ending date) and it has same format and data type as S
        tz_s: is the timezone where the S date was measured in by default library is set to use greenwich(england) time, and it's an integer value
        tz_e: is the same as tz_s in everything, but it's for E date
        count_ls_param: enabling this would count leap-seconds, boolean
        UTDLS: enabling this is useless in all cases except when count_ls_param is True, and it would fetch a list of updated leap seconds from the internet, boolean
        input_utc: this one is by default true, which is when your input is in the UTC system, if it's not make this parameter false
        and your dates would be considered as TAI input, boolean
        output_tt: this one is by default false, which is when you wanna your output to be in TT, boolean
        """
    return td2._time_difference(s, e, tz_s=tz_s, tz_e=tz_e, unit_factor=1, input_utc=input_utc, count_ls_param=count_ls_param, UTDLS=UTDLS, output_tt=output_tt)
def pdminutes (s,e,tz_s=0,tz_e=0,count_ls_param=True,UTDLS=False,input_utc=True,output_tt=False):
    """this function give the difference between the two dates(s and e) in minutes, fractions version
        S: is the first date(starting date) and it's a list made off: [Y(year),M(month),D(day),H(hour),MI(minute),S(second),SS(sub-seconds)] and all of these items integers
        E: is the second date(ending date) and it has same format and data type as S
        tz_s: is the timezone where the S date was measured in by default library is set to use greenwich(england) time, and it's an integer value
        tz_e: is the same as tz_s in everything, but it's for E date
        count_ls_param: enabling this would count leap-seconds, boolean
        UTDLS: enabling this is useless in all cases except when count_ls_param is True, and it would fetch a list of updated leap seconds from the internet, boolean
        input_utc: this one is by default true, which is when your input is in the UTC system, if it's not make this parameter false
        and your dates would be considered as TAI input, boolean
        output_tt: this one is by default false, which is when you wanna your output to be in TT, boolean
        """
    return td1._time_difference(s, e, tz_s=tz_s, tz_e=tz_e, unit_factor=60, input_utc=input_utc, count_ls_param=count_ls_param, UTDLS=UTDLS, output_tt=output_tt)
def cdminutes(s, e, tz_s=0, tz_e=0, count_ls_param=True, UTDLS=False, input_utc=True, output_tt=False):
    """this function give the difference between the two dates(s and e) in minutes, fractionals version
        S: is the first date(starting date) and it's a list made off: [Y(year),M(month),D(day),H(hour),MI(minute),S(second),SS(sub-seconds)] and all of these items integers
        E: is the second date(ending date) and it has same format and data type as S
        tz_s: is the timezone where the S date was measured in by default library is set to use greenwich(england) time, and it's an integer value
        tz_e: is the same as tz_s in everything, but it's for E date
        count_ls_param: enabling this would count leap-seconds, boolean
        UTDLS: enabling this is useless in all cases except when count_ls_param is True, and it would fetch a list of updated leap seconds from the internet, boolean
        input_utc: this one is by default true, which is when your input is in the UTC system, if it's not make this parameter false
        and your dates would be considered as TAI input, boolean
        output_tt: this one is by default false, which is when you wanna your output to be in TT, boolean
        """
    return td2._time_difference(s, e, tz_s=tz_s, tz_e=tz_e, unit_factor=60, input_utc=input_utc, count_ls_param=count_ls_param, UTDLS=UTDLS, output_tt=output_tt)
def pdhours (s,e,tz_s=0,tz_e=0,count_ls_param=True,UTDLS=False,input_utc=True,output_tt=False):
    """this function give the difference between the two dates(s and e) in hours, fractions version
        S: is the first date(starting date) and it's a list made off: [Y(year),M(month),D(day),H(hour),MI(minute),S(second),SS(sub-seconds)] and all of these items integers
        E: is the second date(ending date) and it has same format and data type as S
        tz_s: is the timezone where the S date was measured in by default library is set to use greenwich(england) time, and it's an integer value
        tz_e: is the same as tz_s in everything, but it's for E date
        count_ls_param: enabling this would count leap-seconds, boolean
        UTDLS: enabling this is useless in all cases except when count_ls_param is True, and it would fetch a list of updated leap seconds from the internet, boolean
        input_utc: this one is by default true, which is when your input is in the UTC system, if it's not make this parameter false
        and your dates would be considered as TAI input, boolean
        output_tt: this one is by default false, which is when you wanna your output to be in TT, boolean
        """
    return td1._time_difference(s, e, tz_s=tz_s, tz_e=tz_e, unit_factor=3600, input_utc=input_utc, count_ls_param=count_ls_param, UTDLS=UTDLS, output_tt=output_tt)
def cdhours(s, e, tz_s=0, tz_e=0, count_ls_param=True, UTDLS=False, input_utc=True, output_tt=False):
    """this function give the difference between the two dates(s and e) in hours, fractionals version
        S: is the first date(starting date) and it's a list made off: [Y(year),M(month),D(day),H(hour),MI(minute),S(second),SS(sub-seconds)] and all of these items integers
        E: is the second date(ending date) and it has same format and data type as S
        tz_s: is the timezone where the S date was measured in by default library is set to use greenwich(england) time, and it's an integer value
        tz_e: is the same as tz_s in everything, but it's for E date
        count_ls_param: enabling this would count leap-seconds, boolean
        UTDLS: enabling this is useless in all cases except when count_ls_param is True, and it would fetch a list of updated leap seconds from the internet, boolean
        input_utc: this one is by default true, which is when your input is in the UTC system, if it's not make this parameter false
        and your dates would be considered as TAI input, boolean
        output_tt: this one is by default false, which is when you wanna your output to be in TT, boolean
        """
    return td2._time_difference(s, e, tz_s=tz_s, tz_e=tz_e, unit_factor=3600, input_utc=input_utc, count_ls_param=count_ls_param, UTDLS=UTDLS, output_tt=output_tt)
def pddays (s,e,tz_s=0,tz_e=0,count_ls_param=True,UTDLS=False,input_utc=True,output_tt=False):
    """this function give the difference between the two dates(s and e) in days, fractions version
        S: is the first date(starting date) and it's a list made off: [Y(year),M(month),D(day),H(hour),MI(minute),S(second),SS(sub-seconds)] and all of these items integers
        E: is the second date(ending date) and it has same format and data type as S
        tz_s: is the timezone where the S date was measured in by default library is set to use greenwich(england) time, and it's an integer value
        tz_e: is the same as tz_s in everything, but it's for E date
        count_ls_param: enabling this would count leap-seconds, boolean
        UTDLS: enabling this is useless in all cases except when count_ls_param is True, and it would fetch a list of updated leap seconds from the internet, boolean
        input_utc: this one is by default true, which is when your input is in the UTC system, if it's not make this parameter false
        and your dates would be considered as TAI input, boolean
        output_tt: this one is by default false, which is when you wanna your output to be in TT, boolean
        """
    return td1._time_difference(s, e, tz_s=tz_s, tz_e=tz_e, unit_factor=86400, input_utc=input_utc, count_ls_param=count_ls_param, UTDLS=UTDLS, output_tt=output_tt)
def cddays(s, e, tz_s=0, tz_e=0, count_ls_param=True, UTDLS=False, input_utc=True, output_tt=False):
    """this function give the difference between the two dates(s and e) in seconds, fractionals version
        S: is the first date(starting date) and it's a list made off: [Y(year),M(month),D(day),H(hour),MI(minute),S(second),SS(sub-seconds)] and all of these items integers
        E: is the second date(ending date) and it has same format and data type as S
        tz_s: is the timezone where the S date was measured in by default library is set to use greenwich(england) time, and it's an integer value
        tz_e: is the same as tz_s in everything, but it's for E date
        count_ls_param: enabling this would count leap-seconds, boolean
        UTDLS: enabling this is useless in all cases except when count_ls_param is True, and it would fetch a list of updated leap seconds from the internet, boolean
        input_utc: this one is by default true, which is when your input is in the UTC system, if it's not make this parameter false
        and your dates would be considered as TAI input, boolean
        output_tt: this one is by default false, which is when you wanna your output to be in TT, boolean
        """
    return td2._time_difference(s, e, tz_s=tz_s, tz_e=tz_e, unit_factor=86400, input_utc=input_utc, count_ls_param=count_ls_param, UTDLS=UTDLS, output_tt=output_tt)
def pdweeks (s,e,tz_s=0,tz_e=0,count_ls_param=True,UTDLS=False,input_utc=True,output_tt=False):
    """this function give the difference between the two dates(s and e) in weeks, fractions version
        S: is the first date(starting date) and it's a list made off: [Y(year),M(month),D(day),H(hour),MI(minute),S(second),SS(sub-seconds)] and all of these items integers
        E: is the second date(ending date) and it has same format and data type as S
        tz_s: is the timezone where the S date was measured in by default library is set to use greenwich(england) time, and it's an integer value
        tz_e: is the same as tz_s in everything, but it's for E date
        count_ls_param: enabling this would count leap-seconds, boolean
        UTDLS: enabling this is useless in all cases except when count_ls_param is True, and it would fetch a list of updated leap seconds from the internet, boolean
        input_utc: this one is by default true, which is when your input is in the UTC system, if it's not make this parameter false
        and your dates would be considered as TAI input, boolean
        output_tt: this one is by default false, which is when you wanna your output to be in TT, boolean
        """
    return td1._time_difference(s, e, tz_s=tz_s, tz_e=tz_e, unit_factor=86400 * 7, input_utc=input_utc, count_ls_param=count_ls_param, UTDLS=UTDLS, output_tt=output_tt)
def cdweeks(s, e, tz_s=0, tz_e=0, count_ls_param=True, UTDLS=False, input_utc=True, output_tt=False):
    """this function give the difference between the two dates(s and e) in weeks, fractionals version
        S: is the first date(starting date) and it's a list made off: [Y(year),M(month),D(day),H(hour),MI(minute),S(second),SS(sub-seconds)] and all of these items integers
        E: is the second date(ending date) and it has same format and data type as S
        tz_s: is the timezone where the S date was measured in by default library is set to use greenwich(england) time, and it's an integer value
        tz_e: is the same as tz_s in everything, but it's for E date
        count_ls_param: enabling this would count leap-seconds, boolean
        UTDLS: enabling this is useless in all cases except when count_ls_param is True, and it would fetch a list of updated leap seconds from the internet, boolean
        input_utc: this one is by default true, which is when your input is in the UTC system, if it's not make this parameter false
        and your dates would be considered as TAI input, boolean
        output_tt: this one is by default false, which is when you wanna your output to be in TT, boolean
        """
    return td2._time_difference(s, e, tz_s=tz_s, tz_e=tz_e, unit_factor=86400 * 7, input_utc=input_utc, count_ls_param=count_ls_param, UTDLS=UTDLS, output_tt=output_tt)
def pdmonths(s, e, tz_s=0, tz_e=0,month_unit=0, leap_year=False,input_utc=True, output_tt=False,UTDLS=False, count_ls_param=True,use_date_in_leapyears_checking=False,year_for_leapyears_checking=None,reference_year_for_scaling=None):
    """this function give the difference between the two dates(s and e) in months, fractions version
        S: is the first date(starting date) and it's a list made off: [Y(year),M(month),D(day),H(hour),MI(minute),S(second),SS(sub-seconds)] and all of these items integers
        E: is the second date(ending date) and it has same format and data type as S
        tz_s: is the timezone where the S date was measured in by default library is set to use greenwich(england) time, and it's an integer value
        tz_e: is the same as tz_s in everything, but it's for E date
        month_unit: is what month you chose to be your unit, like how many junes or how many octobers, it's value corresponds to month it represents, it's an integer
        leap_year: if the year is a leap yest or not , useful to determine February's value
        input_utc: this one is by default true, which is when your input is in the UTC system, if it's not make this parameter false
        and your dates would be considered as TAI input, boolean
        output_tt: this one is by default false, which is when you wanna your output to be in TT, boolean
        UTDLS: enabling this is useless in all cases except when count_ls_param is True, and it would fetch a list of updated leap seconds from the internet, boolean
        count_ls_param: enabling this would count leap-seconds, boolean
        use_date_in_leap years_checking: it's an integer value, that would be about to use the year given or written as integer value to determine if year is leap year or not
        year_for_leapyears_checking: it's an integer value, that would be used instead of using the year in starting value to know if to consider
        the year as a leap year or a normal one
        reference_year_for_scaling: it's an boolean value, that would be about if or if not to use the date to determine if year to be used or considered
        is a leap year or not
        """
    return td1._months_piecewise(s, e, tz_s=tz_s, tz_e=tz_e, month_unit=month_unit, leap_year=leap_year, input_utc=input_utc, UTDLS=UTDLS, count_ls_param=count_ls_param, output_tt=output_tt, use_date_in_leapyears_checking=use_date_in_leapyears_checking, year_for_leapyears_checking=year_for_leapyears_checking, reference_year_for_scaling=reference_year_for_scaling)
def cdmonths(s, e, tz_s=0, tz_e=0,month_unit=0, leap_year=False,input_utc=True, output_tt=False,UTDLS=False, count_ls_param=True,use_date_in_leapyears_checking=False,year_for_leapyears_checking=None,reference_year_for_scaling=None):
    """this function give the difference between the two dates(s and e) in months, fractionals version
            S: is the first date(starting date) and it's a list made off: [Y(year),M(month),D(day),H(hour),MI(minute),S(second),SS(sub-seconds)] and all of these items integers
            E: is the second date(ending date) and it has same format and data type as S
            tz_s: is the timezone where the S date was measured in by default library is set to use greenwich(england) time, and it's an integer value
            tz_e: is the same as tz_s in everything, but it's for E date
            month_unit: is what month you chose to be your unit, like how many junes or how many octobers, it's value corresponds to month it represents, it's an integer
            leap_year: if the year is a leap yest or not , useful to determine February's value
            input_utc: this one is by default true, which is when your input is in the UTC system, if it's not make this parameter false
            and your dates would be considered as TAI input, boolean
            output_tt: this one is by default false, which is when you wanna your output to be in TT, boolean
            UTDLS: enabling this is useless in all cases except when count_ls_param is True, and it would fetch a list of updated leap seconds from the internet, boolean
            count_ls_param: enabling this would count leap-seconds, boolean
            use_date_in_leap years_checking: it's an integer value, that would be about to use the year given or written as integer value to determine if year is leap year or not
            year_for_leapyears_checking: it's an integer value, that would be used instead of using the year in starting value to know if to consider
            the year as a leap year or a normal one
            reference_year_for_scaling: it's an boolean value, that would be about if or if not to use the date to determine if year to be used or considered
            is a leap year or not
            """
    return td2._months_piecewise(s, e, tz_s=tz_s, tz_e=tz_e, month_unit=month_unit, leap_year=leap_year, input_utc=input_utc, UTDLS=UTDLS, count_ls_param=count_ls_param, output_tt=output_tt, use_date_in_leapyears_checking=use_date_in_leapyears_checking, year_for_leapyears_checking=year_for_leapyears_checking, reference_year_for_scaling=reference_year_for_scaling)
def pdyears(s, e, tz_s=0, tz_e=0,leap_year=None, uniform_year=False, tropical_year=False, custom_year=None,input_utc=True, output_tt=False,UTDLS=False, count_ls_param=True,year_for_leapyear_checking=None,lock_range_to_reference_year=False):
    """this function give the difference between the two dates(s and e) in years, fractions version
                S: is the first date(starting date) and it's a list made off: [Y(year),M(month),D(day),H(hour),MI(minute),S(second),SS(sub-seconds)] and all of these items integers
                E: is the second date(ending date) and it has same format and data type as S
                tz_s: is the timezone where the S date was measured in by default library is set to use greenwich(england) time, and it's an integer value
                tz_e: is the same as tz_s in everything, but it's for E date
                leap_year: if the year unit is a leap yest or not , boolean value
                uniform_year: if the year unit is a continuous scale Gregorian year, boolean value
                tropical_year: if the year unit is a tropical years, boolean vale
                custom_year: if not none it would use a custom made year unit, integer value
                input_utc: this one is by default true, which is when your input is in the UTC system, if it's not make this parameter false
                and your dates would be considered as TAI input, boolean
                output_tt: this one is by default false, which is when you wanna your output to be in TT, boolean
                UTDLS: enabling this is useless in all cases except when count_ls_param is True, and it would fetch a list of updated leap seconds from the internet, boolean
                count_ls_param: enabling this would count leap-seconds, boolean
                years_for_leapyear_checking: if to use or not use the year in the date to determine if year is a leap year or not, boolean value
                lock_range_to_reference_year: if true no in between years making the added value to pure difference as zero, while default is max(0, E[0] - S[0] - 1)
                """
    return td1._years_piecewise(s, e, tz_s=tz_s, tz_e=tz_e, input_utc=input_utc, UTDLS=UTDLS, count_ls_param=count_ls_param, output_tt=output_tt, uniform_year=uniform_year, tropical_year=tropical_year, custom_year=custom_year, leap_year=leap_year, year_for_leapyear_checking=year_for_leapyear_checking, lock_range_to_reference_year=lock_range_to_reference_year)
def cdyears(s, e, tz_s=0, tz_e=0,leap_year=None, uniform_year=False, tropical_year=False, custom_year=None,input_utc=True, output_tt=False,UTDLS=False, count_ls_param=True,year_for_leapyear_checking=None,lock_range_to_reference_year=False):
    """this function give the difference between the two dates(s and e) in years, fractionals version
                S: is the first date(starting date) and it's a list made off: [Y(year),M(month),D(day),H(hour),MI(minute),S(second),SS(sub-seconds)] and all of these items integers
                E: is the second date(ending date) and it has same format and data type as S
                tz_s: is the timezone where the S date was measured in by default library is set to use greenwich(england) time, and it's an integer value
                tz_e: is the same as tz_s in everything, but it's for E date
                leap_year: if the year unit is a leap yest or not , boolean value
                uniform_year: if the year unit is a continuous scale Gregorian year, boolean value
                tropical_year: if the year unit is a tropical years, boolean vale
                custom_year: if not none it would use a custom made year unit, integer value
                input_utc: this one is by default true, which is when your input is in the UTC system, if it's not make this parameter false
                and your dates would be considered as TAI input, boolean
                output_tt: this one is by default false, which is when you wanna your output to be in TT, boolean
                UTDLS: enabling this is useless in all cases except when count_ls_param is True, and it would fetch a list of updated leap seconds from the internet, boolean
                count_ls_param: enabling this would count leap-seconds, boolean
                years_for_leapyear_checking: if to use or not use the year in the date to determine if year is a leap year or not, boolean value
                lock_range_to_reference_year: if true no in between years making the added value to pure difference as zero, while default is max(0, E[0] - S[0] - 1)
                """
    return td2._years_piecewise(s, e, tz_s=tz_s, tz_e=tz_e, input_utc=input_utc, UTDLS=UTDLS, count_ls_param=count_ls_param, output_tt=output_tt, uniform_year=uniform_year, tropical_year=tropical_year, custom_year=custom_year, leap_year=leap_year, year_for_leapyear_checking=year_for_leapyear_checking, lock_range_to_reference_year=lock_range_to_reference_year)
def pddecades(s, e, tz_s=0, tz_e=0,leap_year=None, uniform_year=False, tropical_year=False, custom_year=None,input_utc=True, output_tt=False,UTDLS=False, count_ls_param=True,year_for_leapyear_checking=None,lock_range_to_reference_year=False):
    """this function give the difference between the two dates(s and e) in decades, fractions version
                    S: is the first date(starting date) and it's a list made off: [Y(year),M(month),D(day),H(hour),MI(minute),S(second),SS(sub-seconds)] and all of these items integers
                    E: is the second date(ending date) and it has same format and data type as S
                    tz_s: is the timezone where the S date was measured in by default library is set to use greenwich(england) time, and it's an integer value
                    tz_e: is the same as tz_s in everything, but it's for E date
                    leap_year: if the year unit is a leap yest or not , boolean value
                    uniform_year: if the year unit is a continuous scale Gregorian year, boolean value
                    tropical_year: if the year unit is a tropical years, boolean vale
                    custom_year: if not none it would use a custom made year unit, integer value
                    input_utc: this one is by default true, which is when your input is in the UTC system, if it's not make this parameter false
                    and your dates would be considered as TAI input, boolean
                    output_tt: this one is by default false, which is when you wanna your output to be in TT, boolean
                    UTDLS: enabling this is useless in all cases except when count_ls_param is True, and it would fetch a list of updated leap seconds from the internet, boolean
                    count_ls_param: enabling this would count leap-seconds, boolean
                    years_for_leapyear_checking: if to use or not use the year in the date to determine if year is a leap year or not, boolean value
                    lock_range_to_reference_year: if true no in between years making the added value to pure difference as zero, while default is max(0, E[0] - S[0] - 1)
                    """
    return td1._years_piecewise(s, e, tz_s=tz_s, tz_e=tz_e, input_utc=input_utc, UTDLS=UTDLS, count_ls_param=count_ls_param, output_tt=output_tt, uniform_year=uniform_year, tropical_year=tropical_year, custom_year=custom_year, leap_year=leap_year, year_for_leapyear_checking=year_for_leapyear_checking, lock_range_to_reference_year=lock_range_to_reference_year)/10
def cddecades(s, e, tz_s=0, tz_e=0,leap_year=None, uniform_year=False, tropical_year=False, custom_year=None,input_utc=True, output_tt=False,UTDLS=False, count_ls_param=True,year_for_leapyear_checking=None,lock_range_to_reference_year=False):
    """this function give the difference between the two dates(s and e) in decades, fractionals version
                    S: is the first date(starting date) and it's a list made off: [Y(year),M(month),D(day),H(hour),MI(minute),S(second),SS(sub-seconds)] and all of these items integers
                    E: is the second date(ending date) and it has same format and data type as S
                    tz_s: is the timezone where the S date was measured in by default library is set to use greenwich(england) time, and it's an integer value
                    tz_e: is the same as tz_s in everything, but it's for E date
                    leap_year: if the year unit is a leap yest or not , boolean value
                    uniform_year: if the year unit is a continuous scale Gregorian year, boolean value
                    tropical_year: if the year unit is a tropical years, boolean vale
                    custom_year: if not none it would use a custom made year unit, integer value
                    input_utc: this one is by default true, which is when your input is in the UTC system, if it's not make this parameter false
                    and your dates would be considered as TAI input, boolean
                    output_tt: this one is by default false, which is when you wanna your output to be in TT, boolean
                    UTDLS: enabling this is useless in all cases except when count_ls_param is True, and it would fetch a list of updated leap seconds from the internet, boolean
                    count_ls_param: enabling this would count leap-seconds, boolean
                    years_for_leapyear_checking: if to use or not use the year in the date to determine if year is a leap year or not, boolean value
                    lock_range_to_reference_year: if true no in between years making the added value to pure difference as zero, while default is max(0, E[0] - S[0] - 1)
                    """
    return td2._years_piecewise(s, e, tz_s=tz_s, tz_e=tz_e, input_utc=input_utc, UTDLS=UTDLS, count_ls_param=count_ls_param, output_tt=output_tt, uniform_year=uniform_year, tropical_year=tropical_year, custom_year=custom_year, leap_year=leap_year, year_for_leapyear_checking=year_for_leapyear_checking, lock_range_to_reference_year=lock_range_to_reference_year)/10
def pdcenturies(s, e, tz_s=0, tz_e=0,leap_year=None, uniform_year=False, tropical_year=False, custom_year=None,input_utc=True, output_tt=False,UTDLS=False, count_ls_param=True,year_for_leapyear_checking=None,lock_range_to_reference_year=False):
    """this function give the difference between the two dates(s and e) in centuries, fractions version
                    S: is the first date(starting date) and it's a list made off: [Y(year),M(month),D(day),H(hour),MI(minute),S(second),SS(sub-seconds)] and all of these items integers
                    E: is the second date(ending date) and it has same format and data type as S
                    tz_s: is the timezone where the S date was measured in by default library is set to use greenwich(england) time, and it's an integer value
                    tz_e: is the same as tz_s in everything, but it's for E date
                    leap_year: if the year unit is a leap yest or not , boolean value
                    uniform_year: if the year unit is a continuous scale Gregorian year, boolean value
                    tropical_year: if the year unit is a tropical years, boolean vale
                    custom_year: if not none it would use a custom made year unit, integer value
                    input_utc: this one is by default true, which is when your input is in the UTC system, if it's not make this parameter false
                    and your dates would be considered as TAI input, boolean
                    output_tt: this one is by default false, which is when you wanna your output to be in TT, boolean
                    UTDLS: enabling this is useless in all cases except when count_ls_param is True, and it would fetch a list of updated leap seconds from the internet, boolean
                    count_ls_param: enabling this would count leap-seconds, boolean
                    years_for_leapyear_checking: if to use or not use the year in the date to determine if year is a leap year or not, boolean value
                    lock_range_to_reference_year: if true no in between years making the added value to pure difference as zero, while default is max(0, E[0] - S[0] - 1)
                    """
    return td1._years_piecewise(s, e, tz_s=tz_s, tz_e=tz_e, input_utc=input_utc, UTDLS=UTDLS, count_ls_param=count_ls_param, output_tt=output_tt, uniform_year=uniform_year, tropical_year=tropical_year, custom_year=custom_year, leap_year=leap_year, year_for_leapyear_checking=year_for_leapyear_checking, lock_range_to_reference_year=lock_range_to_reference_year)/100
def cdcenturies(s, e, tz_s=0, tz_e=0,leap_year=None, uniform_year=False, tropical_year=False, custom_year=None,input_utc=True, output_tt=False,UTDLS=False, count_ls_param=True,year_for_leapyear_checking=None,lock_range_to_reference_year=False):
    """this function give the difference between the two dates(s and e) in centuries, fractionals version
                    S: is the first date(starting date) and it's a list made off: [Y(year),M(month),D(day),H(hour),MI(minute),S(second),SS(sub-seconds)] and all of these items integers
                    E: is the second date(ending date) and it has same format and data type as S
                    tz_s: is the timezone where the S date was measured in by default library is set to use greenwich(england) time, and it's an integer value
                    tz_e: is the same as tz_s in everything, but it's for E date
                    leap_year: if the year unit is a leap yest or not , boolean value
                    uniform_year: if the year unit is a continuous scale Gregorian year, boolean value
                    tropical_year: if the year unit is a tropical years, boolean vale
                    custom_year: if not none it would use a custom made year unit, integer value
                    input_utc: this one is by default true, which is when your input is in the UTC system, if it's not make this parameter false
                    and your dates would be considered as TAI input, boolean
                    output_tt: this one is by default false, which is when you wanna your output to be in TT, boolean
                    UTDLS: enabling this is useless in all cases except when count_ls_param is True, and it would fetch a list of updated leap seconds from the internet, boolean
                    count_ls_param: enabling this would count leap-seconds, boolean
                    years_for_leapyear_checking: if to use or not use the year in the date to determine if year is a leap year or not, boolean value
                    lock_range_to_reference_year: if true no in between years making the added value to pure difference as zero, while default is max(0, E[0] - S[0] - 1)
                    """
    return td2._years_piecewise(s, e, tz_s=tz_s, tz_e=tz_e, input_utc=input_utc, UTDLS=UTDLS, count_ls_param=count_ls_param, output_tt=output_tt, uniform_year=uniform_year, tropical_year=tropical_year, custom_year=custom_year, leap_year=leap_year, year_for_leapyear_checking=year_for_leapyear_checking, lock_range_to_reference_year=lock_range_to_reference_year)/100
def pdmillennia(s, e, tz_s=0, tz_e=0,leap_year=None, uniform_year=False, tropical_year=False, custom_year=None,input_utc=True, output_tt=False,UTDLS=False, count_ls_param=True,year_for_leapyear_checking=None,lock_range_to_reference_year=False):
    """this function give the difference between the two dates(s and e) in millennia, fractions version
                        S: is the first date(starting date) and it's a list made off: [Y(year),M(month),D(day),H(hour),MI(minute),S(second),SS(sub-seconds)] and all of these items integers
                        E: is the second date(ending date) and it has same format and data type as S
                        tz_s: is the timezone where the S date was measured in by default library is set to use greenwich(england) time, and it's an integer value
                        tz_e: is the same as tz_s in everything, but it's for E date
                        leap_year: if the year unit is a leap yest or not , boolean value
                        uniform_year: if the year unit is a continuous scale Gregorian year, boolean value
                        tropical_year: if the year unit is a tropical years, boolean vale
                        custom_year: if not none it would use a custom made year unit, integer value
                        input_utc: this one is by default true, which is when your input is in the UTC system, if it's not make this parameter false
                        and your dates would be considered as TAI input, boolean
                        output_tt: this one is by default false, which is when you wanna your output to be in TT, boolean
                        UTDLS: enabling this is useless in all cases except when count_ls_param is True, and it would fetch a list of updated leap seconds from the internet, boolean
                        count_ls_param: enabling this would count leap-seconds, boolean
                        years_for_leapyear_checking: if to use or not use the year in the date to determine if year is a leap year or not, boolean value
                        lock_range_to_reference_year: if true no in between years making the added value to pure difference as zero, while default is max(0, E[0] - S[0] - 1)
                        """
    return td1._years_piecewise(s, e, tz_s=tz_s, tz_e=tz_e, input_utc=input_utc, UTDLS=UTDLS, count_ls_param=count_ls_param, output_tt=output_tt, uniform_year=uniform_year, tropical_year=tropical_year, custom_year=custom_year, leap_year=leap_year, year_for_leapyear_checking=year_for_leapyear_checking, lock_range_to_reference_year=lock_range_to_reference_year)/1000
def cdmillennia(s, e, tz_s=0, tz_e=0,leap_year=None, uniform_year=False, tropical_year=False, custom_year=None,input_utc=True, output_tt=False,UTDLS=False, count_ls_param=True,year_for_leapyear_checking=None,lock_range_to_reference_year=False):
    """this function give the difference between the two dates(s and e) in millennia, fractionals version
                        S: is the first date(starting date) and it's a list made off: [Y(year),M(month),D(day),H(hour),MI(minute),S(second),SS(sub-seconds)] and all of these items integers
                        E: is the second date(ending date) and it has same format and data type as S
                        tz_s: is the timezone where the S date was measured in by default library is set to use greenwich(england) time, and it's an integer value
                        tz_e: is the same as tz_s in everything, but it's for E date
                        leap_year: if the year unit is a leap yest or not , boolean value
                        uniform_year: if the year unit is a continuous scale Gregorian year, boolean value
                        tropical_year: if the year unit is a tropical years, boolean vale
                        custom_year: if not none it would use a custom made year unit, integer value
                        input_utc: this one is by default true, which is when your input is in the UTC system, if it's not make this parameter false
                        and your dates would be considered as TAI input, boolean
                        output_tt: this one is by default false, which is when you wanna your output to be in TT, boolean
                        UTDLS: enabling this is useless in all cases except when count_ls_param is True, and it would fetch a list of updated leap seconds from the internet, boolean
                        count_ls_param: enabling this would count leap-seconds, boolean
                        years_for_leapyear_checking: if to use or not use the year in the date to determine if year is a leap year or not, boolean value
                        lock_range_to_reference_year: if true no in between years making the added value to pure difference as zero, while default is max(0, E[0] - S[0] - 1)
                        """
    return td2._years_piecewise(s, e, tz_s=tz_s, tz_e=tz_e, input_utc=input_utc, UTDLS=UTDLS, count_ls_param=count_ls_param, output_tt=output_tt, uniform_year=uniform_year, tropical_year=tropical_year, custom_year=custom_year, leap_year=leap_year, year_for_leapyear_checking=year_for_leapyear_checking, lock_range_to_reference_year=lock_range_to_reference_year)/1000
def pdsubseconds(s,e,tz_s=0,tz_e=0,subunit=10**6,count_ls_param=True,UTDLS=False,input_utc=True,output_tt=False):
    """this function give the difference between the two dates(s and e) in subseconds, fractions version
        S: is the first date(starting date) and it's a list made off: [Y(year),M(month),D(day),H(hour),MI(minute),S(second),SS(sub-seconds)] and all of these items integers
        E: is the second date(ending date) and it has same format and data type as S
        tz_s: is the timezone where the S date was measured in by default library is set to use greenwich(england) time, and it's an integer value
        tz_e: is the same as tz_s in everything, but it's for E date
        subunit: basically is the number that seconds would be divided upon to get difference in as specific unit like 10**6 for micro or 10**9 nano seconds, the default is 10**6 and it's an
        count_ls_param: enabling this would count leap-seconds, boolean
        UTDLS: enabling this is useless in all cases except when count_ls_param is True, and it would fetch a list of updated leap seconds from the internet, boolean
        input_utc: this one is by default true, which is when your input is in the UTC system, if it's not make this parameter false
        and your dates would be considered as TAI input, boolean
        output_tt: this one is by default false, which is when you wanna your output to be in TT, boolean
        """
    return td1._time_difference(s, e, tz_s=tz_s, tz_e=tz_e, unit_factor=subunit, input_utc=input_utc, count_ls_param=count_ls_param, UTDLS=UTDLS, output_tt=output_tt)
def cdsubseconds(s,e,tz_s=0,tz_e=0,subunit=10**6,count_ls_param=True,UTDLS=False,input_utc=True,output_tt=False):
    """this function give the difference between the two dates(s and e) in subseconds, fractionals version
            S: is the first date(starting date) and it's a list made off: [Y(year),M(month),D(day),H(hour),MI(minute),S(second),SS(sub-seconds)] and all of these items integers
            E: is the second date(ending date) and it has same format and data type as S
            tz_s: is the timezone where the S date was measured in by default library is set to use greenwich(england) time, and it's an integer value
            tz_e: is the same as tz_s in everything, but it's for E date
            subunit: basically is the number that seconds would be divided upon to get difference in as specific unit like 10**6 for micro or 10**9 nano seconds, the default is 10**6 and it's an
            count_ls_param: enabling this would count leap-seconds, boolean
            UTDLS: enabling this is useless in all cases except when count_ls_param is True, and it would fetch a list of updated leap seconds from the internet, boolean
            input_utc: this one is by default true, which is when your input is in the UTC system, if it's not make this parameter false
            and your dates would be considered as TAI input, boolean
            output_tt: this one is by default false, which is when you wanna your output to be in TT, boolean
            """
    return td2._time_difference(s, e, tz_s=tz_s, tz_e=tz_e, unit_factor=subunit, input_utc=input_utc, count_ls_param=count_ls_param, UTDLS=UTDLS, output_tt=output_tt)
def ptseconds(d,count_ls_param=True, UTDLS=False, input_utc=True,unix_epoch=False,custom_epoch_as_utc=True,custom_epoch_as_tt=False,custom_epoch=None,input_tt=False,output_tt=False): return td1._timestampinner(*d, count_ls_param=count_ls_param, UTDLS=UTDLS, input_utc=input_utc, unix_epoch=unix_epoch, custom_epoch_as_utc=custom_epoch_as_utc, custom_epoch_as_tt=custom_epoch_as_tt, custom_epoch=custom_epoch, input_tt=input_tt, output_tt=output_tt)
def ctseconds(d,count_ls_param=True, UTDLS=False, input_utc=True,unix_epoch=False,custom_epoch_as_utc=True,custom_epoch_as_tt=False,custom_epoch=None,input_tt=False,output_tt=False): return td2._timestampinner(*d, count_ls_param=count_ls_param, UTDLS=UTDLS, input_utc=input_utc, unix_epoch=unix_epoch, custom_epoch_as_utc=custom_epoch_as_utc, custom_epoch_as_tt=custom_epoch_as_tt, custom_epoch=custom_epoch, input_tt=input_tt, output_tt=output_tt)
def ptimestamp(d,count_ls_param=True, UTDLS=False, input_utc=True,unix_epoch=False,custom_epoch_as_utc=True,custom_epoch_as_tt=False,custom_epoch=None,input_tt=False,output_tt=False): return td1._timestampinner(*d, count_ls_param=count_ls_param, UTDLS=UTDLS, input_utc=input_utc, unix_epoch=unix_epoch, custom_epoch_as_utc=custom_epoch_as_utc, custom_epoch_as_tt=custom_epoch_as_tt, custom_epoch=custom_epoch, input_tt=input_tt, output_tt=output_tt)
def ctimestamp(d,count_ls_param=True, UTDLS=False, input_utc=True,unix_epoch=False,custom_epoch_as_utc=True,custom_epoch_as_tt=False,custom_epoch=None,input_tt=False,output_tt=False): return td2._timestampinner(*d, count_ls_param=count_ls_param, UTDLS=UTDLS, input_utc=input_utc, unix_epoch=unix_epoch, custom_epoch_as_utc=custom_epoch_as_utc, custom_epoch_as_tt=custom_epoch_as_tt, custom_epoch=custom_epoch, input_tt=input_tt, output_tt=output_tt)
def ptminutes(d,count_ls_param=True, UTDLS=False, input_utc=True,unix_epoch=False,custom_epoch_as_utc=True,custom_epoch_as_tt=False,custom_epoch=None,input_tt=False,output_tt=False): return td1._timestampinner(*d, count_ls_param=count_ls_param, UTDLS=UTDLS, input_utc=input_utc, unix_epoch=unix_epoch, custom_epoch_as_utc=custom_epoch_as_utc, custom_epoch_as_tt=custom_epoch_as_tt, custom_epoch=custom_epoch, input_tt=input_tt, output_tt=output_tt)/60
def ctminutes(d,count_ls_param=True, UTDLS=False, input_utc=True,unix_epoch=False,custom_epoch_as_utc=True,custom_epoch_as_tt=False,custom_epoch=None,input_tt=False,output_tt=False): return td2._timestampinner(*d, count_ls_param=count_ls_param, UTDLS=UTDLS, input_utc=input_utc, unix_epoch=unix_epoch, custom_epoch_as_utc=custom_epoch_as_utc, custom_epoch_as_tt=custom_epoch_as_tt, custom_epoch=custom_epoch, input_tt=input_tt, output_tt=output_tt)/60
def pthours(d,count_ls_param=True, UTDLS=False, input_utc=True,unix_epoch=False,custom_epoch_as_utc=True,custom_epoch_as_tt=False,custom_epoch=None,input_tt=False,output_tt=False): return td1._timestampinner(*d, count_ls_param=count_ls_param, UTDLS=UTDLS, input_utc=input_utc, unix_epoch=unix_epoch, custom_epoch_as_utc=custom_epoch_as_utc, custom_epoch_as_tt=custom_epoch_as_tt, custom_epoch=custom_epoch, input_tt=input_tt, output_tt=output_tt)/3600
def cthours(d,count_ls_param=True, UTDLS=False, input_utc=True,unix_epoch=False,custom_epoch_as_utc=True,custom_epoch_as_tt=False,custom_epoch=None,input_tt=False,output_tt=False): return td2._timestampinner(*d, count_ls_param=count_ls_param, UTDLS=UTDLS, input_utc=input_utc, unix_epoch=unix_epoch, custom_epoch_as_utc=custom_epoch_as_utc, custom_epoch_as_tt=custom_epoch_as_tt, custom_epoch=custom_epoch, input_tt=input_tt, output_tt=output_tt)/3600
def pcdays(d,count_ls_param=True, UTDLS=False, input_utc=True,unix_epoch=False,custom_epoch_as_utc=True,custom_epoch_as_tt=False,custom_epoch=None,input_tt=False,output_tt=False): return td1._timestampinner(*d, count_ls_param=count_ls_param, UTDLS=UTDLS, input_utc=input_utc, unix_epoch=unix_epoch, custom_epoch_as_utc=custom_epoch_as_utc, custom_epoch_as_tt=custom_epoch_as_tt, custom_epoch=custom_epoch, input_tt=input_tt, output_tt=output_tt)/86400
def ctdays(d,count_ls_param=True, UTDLS=False, input_utc=True,unix_epoch=False,custom_epoch_as_utc=True,custom_epoch_as_tt=False,custom_epoch=None,input_tt=False,output_tt=False): return td2._timestampinner(*d, count_ls_param=count_ls_param, UTDLS=UTDLS, input_utc=input_utc, unix_epoch=unix_epoch, custom_epoch_as_utc=custom_epoch_as_utc, custom_epoch_as_tt=custom_epoch_as_tt, custom_epoch=custom_epoch, input_tt=input_tt, output_tt=output_tt)/86400
def ptweeks(d,count_ls_param=True, UTDLS=False, input_utc=True,unix_epoch=False,custom_epoch_as_utc=True,custom_epoch_as_tt=False,custom_epoch=None,input_tt=False,output_tt=False): return td1._timestampinner(*d, count_ls_param=count_ls_param, UTDLS=UTDLS, input_utc=input_utc, unix_epoch=unix_epoch, custom_epoch_as_utc=custom_epoch_as_utc, custom_epoch_as_tt=custom_epoch_as_tt, custom_epoch=custom_epoch, input_tt=input_tt, output_tt=output_tt)/(86400 * 7)
def ctweeks(d,count_ls_param=True, UTDLS=False, input_utc=True,unix_epoch=False,custom_epoch_as_utc=True,custom_epoch_as_tt=False,custom_epoch=None,input_tt=False,output_tt=False): return td2._timestampinner(*d, count_ls_param=count_ls_param, UTDLS=UTDLS, input_utc=input_utc, unix_epoch=unix_epoch, custom_epoch_as_utc=custom_epoch_as_utc, custom_epoch_as_tt=custom_epoch_as_tt, custom_epoch=custom_epoch, input_tt=input_tt, output_tt=output_tt)/(86400 * 7)
def ptmonths(d,count_ls_param=True, UTDLS=False, input_utc=True,unix_epoch=False,custom_epoch_as_utc=True,custom_epoch_as_tt=False,custom_epoch=None,input_tt=False,output_tt=False,month_unit=0,use_date_in_leapyears_checking=False,leap_year=False,year_for_leapyears_checking=None): return td1._timestampinner(*d, count_ls_param=count_ls_param, UTDLS=UTDLS, input_utc=input_utc, unix_epoch=unix_epoch, custom_epoch_as_utc=custom_epoch_as_utc, custom_epoch_as_tt=custom_epoch_as_tt, custom_epoch=custom_epoch, input_tt=input_tt, output_tt=output_tt) / td1._get_month_seconds(d[0], d[1], month_unit, td1.is_leap((d[0] if use_date_in_leapyears_checking else year_for_leapyears_checking), leap_year))
def ctmonths(d,count_ls_param=True, UTDLS=False, input_utc=True,unix_epoch=False,custom_epoch_as_utc=True,custom_epoch_as_tt=False,custom_epoch=None,input_tt=False,output_tt=False,month_unit=0,use_date_in_leapyears_checking=False,leap_year=False,year_for_leapyears_checking=None): return td2._timestampinner(*d, count_ls_param=count_ls_param, UTDLS=UTDLS, input_utc=input_utc, unix_epoch=unix_epoch, custom_epoch_as_utc=custom_epoch_as_utc, custom_epoch_as_tt=custom_epoch_as_tt, custom_epoch=custom_epoch, input_tt=input_tt, output_tt=output_tt) / td2._get_month_seconds(d[0], d[1], month_unit, td2.is_leap((d[0] if use_date_in_leapyears_checking else year_for_leapyears_checking), leap_year))
def ptyears(d, count_ls_param=True, UTDLS=False, input_utc=True, unix_epoch=False, custom_epoch_as_utc=True, custom_epoch_as_tt=False,custom_epoch=None, input_tt=False, output_tt=False,uniform_year=False, tropical_year=False, custom_year=None, year_for_leapyear_checking=None): return td1._timestampinner(*d, count_ls_param=count_ls_param, UTDLS=UTDLS, input_utc=input_utc, unix_epoch=unix_epoch, custom_epoch_as_utc=custom_epoch_as_utc, custom_epoch_as_tt=custom_epoch_as_tt, custom_epoch=custom_epoch, input_tt=input_tt, output_tt=output_tt)/ td1._seconds_in_year(d[0] if year_for_leapyear_checking is None else year_for_leapyear_checking, uniform_year=uniform_year, tropical_year=tropical_year, custom_year=custom_year)
def ctyears(d, count_ls_param=True, UTDLS=False, input_utc=True, unix_epoch=False, custom_epoch_as_utc=True, custom_epoch_as_tt=False,custom_epoch=None, input_tt=False, output_tt=False,uniform_year=False, tropical_year=False, custom_year=None, year_for_leapyear_checking=None): return td2._timestampinner(*d, count_ls_param=count_ls_param, UTDLS=UTDLS, input_utc=input_utc, unix_epoch=unix_epoch, custom_epoch_as_utc=custom_epoch_as_utc, custom_epoch_as_tt=custom_epoch_as_tt, custom_epoch=custom_epoch, input_tt=input_tt, output_tt=output_tt)/ td2._seconds_in_year(d[0] if year_for_leapyear_checking is None else year_for_leapyear_checking, uniform_year=uniform_year, tropical_year=tropical_year, custom_year=custom_year)
def ptdecades(d, count_ls_param=True, UTDLS=False, input_utc=True, unix_epoch=False, custom_epoch_as_utc=True, custom_epoch_as_tt=False,custom_epoch=None, input_tt=False, output_tt=False,uniform_year=False, tropical_year=False, custom_year=None, year_for_leapyear_checking=None): return (
            td1._timestampinner(*d, count_ls_param=count_ls_param, UTDLS=UTDLS, input_utc=input_utc, unix_epoch=unix_epoch, custom_epoch_as_utc=custom_epoch_as_utc, custom_epoch_as_tt=custom_epoch_as_tt, custom_epoch=custom_epoch, input_tt=input_tt, output_tt=output_tt) / td1._seconds_in_year(d[0] if year_for_leapyear_checking is None else year_for_leapyear_checking, uniform_year=uniform_year, tropical_year=tropical_year, custom_year=custom_year))/10
def ctdecades(d, count_ls_param=True, UTDLS=False, input_utc=True, unix_epoch=False, custom_epoch_as_utc=True, custom_epoch_as_tt=False,custom_epoch=None, input_tt=False, output_tt=False,uniform_year=False, tropical_year=False, custom_year=None, year_for_leapyear_checking=None): return (
            td2._timestampinner(*d, count_ls_param=count_ls_param, UTDLS=UTDLS, input_utc=input_utc, unix_epoch=unix_epoch, custom_epoch_as_utc=custom_epoch_as_utc, custom_epoch_as_tt=custom_epoch_as_tt, custom_epoch=custom_epoch, input_tt=input_tt, output_tt=output_tt) / td2._seconds_in_year(d[0] if year_for_leapyear_checking is None else year_for_leapyear_checking, uniform_year=uniform_year, tropical_year=tropical_year, custom_year=custom_year))/10
def ptcenturies(d, count_ls_param=True, UTDLS=False, input_utc=True, unix_epoch=False, custom_epoch_as_utc=True, custom_epoch_as_tt=False,custom_epoch=None, input_tt=False, output_tt=False,uniform_year=False, tropical_year=False, custom_year=None, year_for_leapyear_checking=None): return (
            td1._timestampinner(*d, count_ls_param=count_ls_param, UTDLS=UTDLS, input_utc=input_utc, unix_epoch=unix_epoch, custom_epoch_as_utc=custom_epoch_as_utc, custom_epoch_as_tt=custom_epoch_as_tt, custom_epoch=custom_epoch, input_tt=input_tt, output_tt=output_tt) / td1._seconds_in_year(d[0] if year_for_leapyear_checking is None else year_for_leapyear_checking, uniform_year=uniform_year, tropical_year=tropical_year, custom_year=custom_year))/100
def ctcenturies(d, count_ls_param=True, UTDLS=False, input_utc=True, unix_epoch=False, custom_epoch_as_utc=True, custom_epoch_as_tt=False,custom_epoch=None, input_tt=False, output_tt=False,uniform_year=False, tropical_year=False, custom_year=None, year_for_leapyear_checking=None): return (
            td2._timestampinner(*d, count_ls_param=count_ls_param, UTDLS=UTDLS, input_utc=input_utc, unix_epoch=unix_epoch, custom_epoch_as_utc=custom_epoch_as_utc, custom_epoch_as_tt=custom_epoch_as_tt, custom_epoch=custom_epoch, input_tt=input_tt, output_tt=output_tt) / td2._seconds_in_year(d[0] if year_for_leapyear_checking is None else year_for_leapyear_checking, uniform_year=uniform_year, tropical_year=tropical_year, custom_year=custom_year))/100
def ptmillennia(d, count_ls_param=True, UTDLS=False, input_utc=True, unix_epoch=False, custom_epoch_as_utc=True, custom_epoch_as_tt=False,custom_epoch=None, input_tt=False, output_tt=False,uniform_year=False, tropical_year=False, custom_year=None, year_for_leapyear_checking=None): return (
            td1._timestampinner(*d, count_ls_param=count_ls_param, UTDLS=UTDLS, input_utc=input_utc, unix_epoch=unix_epoch, custom_epoch_as_utc=custom_epoch_as_utc, custom_epoch_as_tt=custom_epoch_as_tt, custom_epoch=custom_epoch, input_tt=input_tt, output_tt=output_tt) / td1._seconds_in_year(d[0] if year_for_leapyear_checking is None else year_for_leapyear_checking, uniform_year=uniform_year, tropical_year=tropical_year, custom_year=custom_year))/1000
def ctmillennia(d, count_ls_param=True, UTDLS=False, input_utc=True, unix_epoch=False, custom_epoch_as_utc=True, custom_epoch_as_tt=False,custom_epoch=None, input_tt=False, output_tt=False,uniform_year=False, tropical_year=False, custom_year=None, year_for_leapyear_checking=None): return (
            td2._timestampinner(*d, count_ls_param=count_ls_param, UTDLS=UTDLS, input_utc=input_utc, unix_epoch=unix_epoch, custom_epoch_as_utc=custom_epoch_as_utc, custom_epoch_as_tt=custom_epoch_as_tt, custom_epoch=custom_epoch, input_tt=input_tt, output_tt=output_tt) / td2._seconds_in_year(d[0] if year_for_leapyear_checking is None else year_for_leapyear_checking, uniform_year=uniform_year, tropical_year=tropical_year, custom_year=custom_year))/1000
def ptsubseconds(d,subunit=10**6,count_ls_param=True, UTDLS=False, input_utc=True,unix_epoch=False,custom_epoch_as_utc=True,custom_epoch_as_tt=False,custom_epoch=None,input_tt=False,output_tt=False): return td1._timestampinner(*d, count_ls_param=count_ls_param, UTDLS=UTDLS, input_utc=input_utc, unix_epoch=unix_epoch, custom_epoch_as_utc=custom_epoch_as_utc, custom_epoch_as_tt=custom_epoch_as_tt, custom_epoch=custom_epoch, input_tt=input_tt, output_tt=output_tt)/(1 / subunit)
def ctsubseconds(d,subunit=10**6,count_ls_param=True, UTDLS=False, input_utc=True,unix_epoch=False,custom_epoch_as_utc=True,custom_epoch_as_tt=False,custom_epoch=None,input_tt=False,output_tt=False): return td2._timestampinner(*d, count_ls_param=count_ls_param, UTDLS=UTDLS, input_utc=input_utc, unix_epoch=unix_epoch, custom_epoch_as_utc=custom_epoch_as_utc, custom_epoch_as_tt=custom_epoch_as_tt, custom_epoch=custom_epoch, input_tt=input_tt, output_tt=output_tt)/(1 / subunit)
def pmonthscal(s,e): return  td1._monthscalner(s, e)
def cmonthscal(s,e): return  td2._monthscalner(s, e)
def pyearscal(s,e): return  td1._yearscalner(s, e)
def cyearscal(s,e): return  td2._yearscalner(s, e)


# fractionals.pyx
# distutils: language=c++
# cython: language_level=3

from decimal import Decimal, localcontext
from fractionals cimport *

cdef class Fraction:

    def __cinit__(self):
        mpq_init(self.value)

    def __dealloc__(self):
        mpq_clear(self.value)

    # -----------------------
    # Helper: set fraction from Python ints (arbitrary size)
    # -----------------------
    cdef void _set_from_bigints(self, object num, object den):
        cdef mpz_t num_mpz, den_mpz
        mpz_init(num_mpz)
        mpz_init(den_mpz)
        mpz_set_str(num_mpz, str(num).encode(), 10)
        mpz_set_str(den_mpz, str(den).encode(), 10)
        mpq_set_num(self.value, num_mpz)
        mpq_set_den(self.value, den_mpz)
        mpq_canonicalize(self.value)
        mpz_clear(num_mpz)
        mpz_clear(den_mpz)

    # -----------------------
    # Initialize from Python object
    # -----------------------
    cdef void _set_from_object(self, object a):
        cdef Fraction bf

        if isinstance(a, int):
            self._set_from_bigints(a, 1)

        elif isinstance(a, float):
            s = format(a, ".50g")
            mpq_set_str(self.value, s.encode(), 10)
            mpq_canonicalize(self.value)

        elif isinstance(a, Decimal):
            try:
                numerator, denominator = a.as_integer_ratio()
                self._set_from_bigints(numerator, denominator)
            except OverflowError:
                s = format(a, "f")
                if '.' in s:
                    integer_part, frac_part = s.split('.')
                    numerator_str = integer_part + frac_part
                    denominator_str = '1' + '0' * len(frac_part)
                else:
                    numerator_str = s
                    denominator_str = '1'
                self._set_from_bigints(numerator_str, denominator_str)

        elif isinstance(a, str):
            if '/' in a:
                parts = a.split('/')
                if len(parts) != 2:
                    raise ValueError(f"Invalid fraction string: {a}")
                self._set_from_bigints(parts[0].strip(), parts[1].strip())
            else:
                mpq_set_str(self.value, a.encode(), 10)
                mpq_canonicalize(self.value)

        elif isinstance(a, Fraction):
            bf = <Fraction>a
            mpq_set(self.value, bf.value)

        else:
            raise TypeError(f"Cannot convert type {type(a)} to Fraction")

    # -----------------------
    # Constructor
    # -----------------------
    def __init__(self, *args):
        if len(args) == 0:
            mpq_set_si(self.value, 0, 1)
        elif len(args) == 1:
            self._set_from_object(args[0])
        elif len(args) == 2:
            num, den = args
            if den == 0:
                raise ZeroDivisionError("Denominator cannot be zero")
            self._set_from_bigints(num, den)
        else:
            raise TypeError("Fraction() takes 0, 1, or 2 arguments")

    # -----------------------
    # Arithmetic helpers
    # -----------------------
    cdef Fraction _binary_op(self, object other,
                             void (*op)(mpq_t, mpq_srcptr, mpq_srcptr)):
        cdef Fraction result = Fraction()
        cdef Fraction o
        if isinstance(other, Fraction):
            o = <Fraction>other
        else:
            o = Fraction(other)
        op(result.value, self.value, o.value)
        return result

    def __add__(self, other): return self._binary_op(other, mpq_add)
    def __sub__(self, other): return self._binary_op(other, mpq_sub)
    def __mul__(self, other): return self._binary_op(other, mpq_mul)
    def __truediv__(self, other): return self._binary_op(other, mpq_div)

    def __radd__(self, other): return Fraction(other) + self
    def __rsub__(self, other): return Fraction(other) - self
    def __rmul__(self, other): return Fraction(other) * self
    def __rtruediv__(self, other): return Fraction(other) / self

    # -----------------------
    # Comparisons
    # -----------------------
    cdef int _cmp(self, object other):
        cdef Fraction o
        if isinstance(other, Fraction):
            o = <Fraction>other
        else:
            o = Fraction(other)
        return mpq_cmp(self.value, o.value)

    def __eq__(self, other): return self._cmp(other) == 0
    def __ne__(self, other): return self._cmp(other) != 0
    def __lt__(self, other): return self._cmp(other) < 0
    def __le__(self, other): return self._cmp(other) <= 0
    def __gt__(self, other): return self._cmp(other) > 0
    def __ge__(self, other): return self._cmp(other) >= 0

    # -----------------------
    # String / representation
    # -----------------------
    def __str__(self):
        cdef char* s = mpq_get_str(NULL, 10, self.value)
        py_str = s.decode()
        free(s)
        return py_str

    def __repr__(self):
        return f"Fraction('{str(self)}')"

    # -----------------------
    # Numerator / Denominator
    # -----------------------
    @property
    def numerator(self):
        cdef mpz_ptr num = mpq_numref(self.value)
        cdef char* s = mpz_get_str(NULL, 10, num)
        py_str = s.decode()
        free(s)
        return int(py_str)

    @property
    def denominator(self):
        cdef mpz_ptr den = mpq_denref(self.value)
        cdef char* s = mpz_get_str(NULL, 10, den)
        py_str = s.decode()
        free(s)
        return int(py_str)

    # -----------------------
    # Conversion
    # -----------------------
    def __float__(self):
        return self.numerator / self.denominator

    def __int__(self):
        return self.numerator // self.denominator

    def __round__(self, n=0):
        return round(float(self), n)

    def __trunc__(self):
        return self.numerator // self.denominator

    def as_integer_ratio(self):
        return (self.numerator, self.denominator)

    def __bool__(self):
        return self.numerator != 0

    def __hash__(self):
        return hash((self.numerator, self.denominator))

    def to_decimal(self, precision=None):
        if precision is None:
            return Decimal(self.numerator) / Decimal(self.denominator)
        else:
            with localcontext() as ctx:
                ctx.prec = precision
                return Decimal(self.numerator) / Decimal(self.denominator)



# fractionals.pxd
# Cython definitions for GMP and Fraction class

from libc.stdlib cimport free

cdef extern from "gmp.h":
    # GMP rational
    ctypedef struct __mpq_struct: pass
    ctypedef __mpq_struct mpq_t[1]
    ctypedef __mpq_struct* mpq_ptr
    ctypedef const __mpq_struct* mpq_srcptr

    # GMP integer
    ctypedef struct __mpz_struct: pass
    ctypedef __mpz_struct mpz_t[1]
    ctypedef __mpz_struct* mpz_ptr
    ctypedef const __mpz_struct* mpz_srcptr

    # mpq functions
    void mpq_init(mpq_t x)
    void mpq_clear(mpq_t x)
    void mpq_set(mpq_t rop, mpq_srcptr op)
    void mpq_set_si(mpq_t rop, long num, unsigned long den)
    int mpq_set_str(mpq_t rop, const char* str, int base)
    void mpq_canonicalize(mpq_t rop)
    int mpq_cmp(mpq_t op1, mpq_t op2)
    void mpq_add(mpq_t rop, mpq_srcptr op1, mpq_srcptr op2)
    void mpq_sub(mpq_t rop, mpq_srcptr op1, mpq_srcptr op2)
    void mpq_mul(mpq_t rop, mpq_srcptr op1, mpq_srcptr op2)
    void mpq_div(mpq_t rop, mpq_srcptr op1, mpq_srcptr op2)
    char* mpq_get_str(char* str, int base, mpq_t op)
    mpz_ptr mpq_numref(mpq_t op)
    mpz_ptr mpq_denref(mpq_t op)
    char* mpz_get_str(char* str, int base, mpz_t op)
    void mpq_set_num(mpq_t rop, mpz_t num)
    void mpq_set_den(mpq_t rop, mpz_t den)

    # mpz functions for big ints
    void mpz_init(mpz_t x)
    void mpz_clear(mpz_t x)
    int mpz_set_str(mpz_t rop, const char* str, int base)

# Fraction class definition for Cython
cdef class Fraction:
    cdef mpq_t value
    cdef void _set_from_bigints(self, object num, object den)
    cdef void _set_from_object(self, object a)
    cdef Fraction _binary_op(self, object other, void (*op)(mpq_t, mpq_srcptr, mpq_srcptr))
    cdef int _cmp(self, object other)
